#!/usr/bin/python3
import socket
import threading
import ssl
import sys
import pprint

SERVERHOST = 'www.python.org'
SERVERPORT = 443

FAKECERT ='fake.crt'
FAKEKEY = 'fake.key'

PROXYHOST = ''
PROXYPORT = int(sys.argv[1])
BUFSIZE = 4096
TIMEOUT = 2.0

proxysocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
proxysocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
proxysocket.bind((PROXYHOST, PROXYPORT))
proxysocket.listen()
proxyaddr = proxysocket.getsockname()

def mythread(clientconn, clientaddr):
    serveraddr = (SERVERHOST, SERVERPORT)
    serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    servercontext = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
    serverconnssl = servercontext.wrap_socket(serversocket, server_hostname=SERVERHOST)
    serverconnssl.connect(serveraddr)
    print('=== Proxy connected to Server:', serveraddr, '===')
    # ssl.get_server_certificate(addr, ssl_version=PROTOCOL_SSLv23, ca_certs=None)

    print("=== Server Cert ===")
    servercert = serverconnssl.getpeercert()
    pprint.pprint(servercert)

    # the fake cert for 'www.python.org' should be generated dynamically!
    proxycontext = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
    proxycontext.load_cert_chain(certfile=FAKECERT, keyfile=FAKEKEY)
    clientconnssl = proxycontext.wrap_socket(clientconn, server_side=True)
    print('=== Client connected to Proxy:', clientaddr, '===')

    # print("=== Fake Cert ===")
    # fakecertpem = ssl.get_server_certificate(proxyaddr)
    # pprint.pprint(fakecert)

    # recv request from client and forward it
    print('=== Request ===')
    request = clientconnssl.recv(BUFSIZE)
    print(request.decode())
    serverconnssl.sendall(request)

    # recv answer from server and forward it to client
    answer = b''
    while True:
        try:
            serverconnssl.settimeout(TIMEOUT) # required to avoid waiting HTTP Keep-Alive timeout!
            data = serverconnssl.recv(BUFSIZE)
            answer += data
            clientconnssl.sendall(data)
            if data == b'': break
        except socket.timeout: break

    print('=== Answer', len(answer), 'bytes ===')
    # print(answer.decode('iso-8859-1'))
    print(answer.decode('utf-8'))

    print('=== Proxy disconnected:', clientaddr, '=>', serveraddr, '===')
    serverconnssl.close()
    clientconnssl.close()

print('=== Proxy listening:', proxyaddr, '===')
while True:
    clientconn, clientaddr = proxysocket.accept()
    print('accept client conn', clientaddr, clientconn)
    if clientconn: threading.Thread(target=mythread,args=(clientconn, clientaddr)).start()

proxysocket.close()
